@@ -4,7 +4,7 @@ use warnings;
 use Carp ();
 use Time::HiRes ();
 
-our $VERSION = '0.10_01';
+our $VERSION = '0.01';
 
 require Dumbbench::Result;
 require Dumbbench::Stats;
@@ -21,7 +21,6 @@ use Class::XSAccessor {
     variability_measure
     started
     outlier_rejection
-    subtract_dry_run
   )],
   accessors => [qw(verbosity)],
 };
@@ -41,8 +40,7 @@ sub new {
       variability_measure  => 'mad',
       instances            => [],
       started              => 0,
-      outlier_rejection    => 3,
-      subtract_dry_run     => 1,
+      outlier_rejection    => 2.5,
       @_,
     } => $class;
   }
@@ -54,20 +52,20 @@ sub new {
       push @{$self->{instances}}, $instance->new;
     }
   }
-
+  
   if ($self->target_abs_precision <= 0 and $self->target_rel_precision <= 0) {
     Carp::croak("Need either target_rel_precision or target_abs_precision > 0");
   }
   if ($self->initial_runs < 6) {
     Carp::carp("Number of initial runs is very small (<6). Precision will be off.");
   }
-
+  
   return $self;
 }
 
 sub add_instances {
   my $self = shift;
-
+  
   if ($self->started) {
     Carp::croak("Can't add instances after the benchmark has been started");
   }
@@ -87,7 +85,7 @@ sub instances {
 sub run {
   my $self = shift;
   Carp::croak("Can't re-run same benchmark instance") if $self->started;
-  $self->dry_run_timings if $self->subtract_dry_run;
+  $self->dry_run_timings;
   $self->run_timings;
 }
 
@@ -115,8 +113,6 @@ sub _run {
   my $instance = shift;
   my $dry = shift;
 
-  my $name = $instance->_name_prefix;
-
   # for overriding in case of dry-run mode
   my $V = $self->verbosity || 0;
   my $initial_timings = $self->initial_runs;
@@ -132,7 +128,7 @@ sub _run {
     $max_iterations  *= 10;
   }
 
-  print "${name}Running initial timing for warming up the cache...\n" if $V;
+  print "Running initial timing for warming up the cache...\n" if $V;
   if ($dry) {
     # be generous, this is fast
     $instance->single_dry_run() for 1..3;
@@ -140,15 +136,15 @@ sub _run {
   else {
     $instance->single_run();
   }
-
+  
   my @timings;
-  print "${name}Running $initial_timings initial timings...\n" if $V;
+  print "Running $initial_timings initial timings...\n" if $V;
   foreach (1..$initial_timings) {
-    print "${name}Running timing $_...\n" if $V > 1;
+    print "Running timing $_...\n" if $V > 1;
     push @timings, ($dry ? $instance->single_dry_run() : $instance->single_run());
   }
 
-  print "${name}Iterating until target precision reached...\n" if $V;
+  print "Iterating until target precision reached...\n" if $V;
 
   my $stats = Dumbbench::Stats->new(data => \@timings);
   my $sigma;
@@ -168,55 +164,51 @@ sub _run {
   my $n_good = 0;
   my $variability_measure = $self->variability_measure;
   while (1) {
-    my ($good, $outliers) = $stats->filter_outliers(
-      variability_measure => $variability_measure,
-      nsigma_outliers     => $self->outlier_rejection,
-    );
+    $sigma = $stats->$variability_measure();# / sqrt(scalar(@timings));
+    $mean  = $stats->mean();
+    my $median = $stats->median();
+    my $outlier_rejection = $self->outlier_rejection;
+    my @t;
+    if ($outlier_rejection) {
+      @t = grep {abs($_-$median) < $outlier_rejection*$sigma} @timings;
+    }
+    else {
+      @t = @timings; # doh
+    }
+    $n_good = @t;
 
-    $n_good = @$good;
-
-    if (not $n_good and @timings >= $max_iterations) {
+    if (not $n_good and @timings == $max_iterations) {
       $mean = 0; $sigma = 0;
       last;
     }
-
+    
     if ($n_good) {
-      my $new_stats = Dumbbench::Stats->new(data => $good);
-      $sigma = $new_stats->$variability_measure() / sqrt($n_good);
+      my $new_stats = Dumbbench::Stats->new(data => \@t);
+      $sigma = $new_stats->$variability_measure() / sqrt(scalar(@t));
       $mean = $new_stats->mean();
 
       # stop condition
       my $need_iter = 0;
       if ($rel_precision > 0) {
         my $rel = $sigma/$mean;
-        print "${name}Reached relative precision $rel (neeed $rel_precision).\n" if $V > 1;
+        print "Reached relative precision $rel (neeed $rel_precision).\n" if $V > 1;
         $need_iter++ if $rel > $rel_precision;
       }
       if ($abs_precision > 0) {
-        print "${name}Reached absolute precision $sigma (neeed $abs_precision).\n" if $V > 1;
+        print "Reached absolute precision $sigma (neeed $abs_precision).\n" if $V > 1;
         $need_iter++ if $sigma > $abs_precision;
       }
       if ($n_good < $initial_timings) {
         $need_iter++;
       }
-      last if not $need_iter or @timings >= $max_iterations;
+      last if not $need_iter or @timings == $max_iterations;
     }
 
-    # progressively run more new timings in one go. Otherwise,
-    # we start to stall on the O(n*log(n)) complexity of the median.
-    my $n = List::Util::min( $max_iterations - @timings, List::Util::max(1, @timings*0.05) );
-    push @timings, ($dry ? $instance->single_dry_run() : $instance->single_run()) for 1..$n;
+    push @timings, ($dry ? $instance->single_dry_run() : $instance->single_run());
   } # end while more data required
 
-  if (@timings >= $max_iterations and not $dry) {
-    print "${name}Reached maximum number of iterations. Stopping. Precision not reached.\n";
-  }
-
-  # rescale sigma
-  # This is necessary since by cutting everything outside of n-sigma,
-  # we artificially reduce the variability of the main distribution.
-  if ($self->outlier_rejection) {
-    # TODO implement
+  if (@timings == $max_iterations and not $dry) {
+    print "Reached maximum number of iterations. Stopping. Precision not reached.\n";
   }
 
   my $result = Dumbbench::Result->new(
@@ -231,8 +223,7 @@ sub _run {
   }
   else {
     $instance->{timings} = \@timings;
-    $result -= $instance->dry_result
-      if defined $instance->dry_result and $self->subtract_dry_run;
+    $result -= $instance->dry_result if defined $instance->dry_result;
     $instance->result($result);
   }
 }
@@ -240,56 +231,30 @@ sub _run {
 sub report {
   my $self = shift;
   my $raw = shift;
-  my $simple = shift;
   foreach my $instance ($self->instances) {
     my $result = $instance->result;
-    my $result_str = ($simple) ? unscientific_notation($result) : "$result";
-
+    
     if (not $raw) {
       my $mean = $result->raw_number;
       my $sigma = $result->raw_error->[0];
-      my $name = $instance->_name_prefix;
-      printf(
-        "%sRan %u iterations (%u outliers).\n",
-        $name,
-        scalar(@{$instance->timings}),
-        scalar(@{$instance->timings})-$result->nsamples
-      );
-      printf(
-        "%sRounded run time per iteration: %s (%.1f%%)\n",
-        $name,
-        $result_str,
-        $sigma/$mean*100
-      );
-      if ($self->verbosity) {
-        printf("%sRaw:                            $mean +/- $sigma\n", $name);
-      }
+      print "Ran " . scalar(@{$instance->timings}) . " iterations of the command.\n";
+      print "Rejected " . (scalar(@{$instance->timings})-$result->nsamples) . " samples as outliers.\n";
+      print "Rounded run time per iteration: $result" . sprintf(" (%.1f%%)\n", $sigma/$mean*100);
+      print "Raw:                            $mean +/- $sigma\n" if $self->verbosity;
     }
     else {
-      print $result_str, "\n";
+      print $result, "\n";
     }
   }
 }
 
-sub box_plot {
-  my $self = shift;
-  eval "require Dumbbench::BoxPlot;";
-  return() if $@;
-
-  return Dumbbench::BoxPlot->new($self);
-}
-
-sub unscientific_notation {
-  sprintf( "%f %s %f", split( / /, $_[0] ) );
-}
-
 1;
 
 __END__
 
 =head1 NAME
 
-Dumbbench - More reliable benchmarking with the least amount of thinking
+Dumbbench - Perl extension more reliable benchmarking
 
 =head1 SYNOPSIS
 
@@ -306,13 +271,13 @@ This will start churning for a while and then prints something like:
 As a module:
 
   use Dumbbench;
-
+  
   my $bench = Dumbbench->new(
     target_rel_precision => 0.005, # seek ~0.5%
     initial_runs         => 20,    # the higher the more reliable
   );
   $bench->add_instances(
-    Dumbbench::Instance::Cmd->new(command => [qw(perl -e 'something')]),
+    Dumbbench::Instance::Cmd->new(command => [qw(perl -e 'something')]), 
     Dumbbench::Instance::PerlEval->new(code => 'for(1..1e7){something}'),
     Dumbbench::Instance::PerlSub->new(code => sub {for(1..1e7){something}}),
   );
@@ -341,9 +306,6 @@ the resulting uncertainty and goes through some pain to discard bad runs and sub
 overhead from the timings. The reported timing includes an uncertainty, so that multiple
 benchmarks can more easily be compared.
 
-Please note that C<Dumbbench> works entirely with wallclock time as reported by
-C<Time::HiRes>' C<time()> function.
-
 =head1 METHODS
 
 In addition to the methods listed here, there are read-only
@@ -360,11 +322,10 @@ Constructor that takes the following arguments (with defaults):
   intial_runs          => 20,    # no. of guaranteed initial runs
   max_iterations       => 10000, # hard max. no of iterations
   variability_measure  => 'mad', # method for calculating uncertainty
-  outlier_rejection    => 3,     # no. of "sigma"s for the outlier rejection
+  outlier_rejection    => 2.5,   # no. of "sigma"s for the outlier rejection
 
 C<variability_measure> and C<outlier_rejection> probably make sense
-after reading C<HOW IT WORKS> below. Setting C<outlier_rejection> to 0
-will turn it off entirely.
+after reading C<HOW IT WORKS> below.
 
 =head2 add_instances
 
@@ -394,18 +355,6 @@ Returns a list of all instance objects in this benchmark set.
 The instance objects each have a C<result()> and C<dry_result()>
 method for accessing the numeric benchmark results.
 
-=head2 box_plot
-
-Returns a L<Dumbbench::BoxPlot> instance.
-
-A L<Dumbbench::BoxPlot> is a nice an easy way to get a graphic chart if
-you're in the mood instead of getting the same results from C<report>.
-
-If you don't want to get into the details of L<Dumbbench::BoxPlot>, you can do:
-
-  # $bench is your Dumbbench instance
-  $bench->box_plot->show;
-
 =head1 HOW IT WORKS AND WHY IT DOESN'T
 
 =head2 Why it doesn't work and why we try regardless
@@ -500,8 +449,6 @@ value and the MAD of the truncated distribution as a measure of variability.
 To get the uncertainty on the expectation value, we take C<MAD / sqrt($N)> where
 C<$N> is the number of remaining measurements.
 
-=back
-
 =head2 Conclusion
 
 I hope I could convince you that interpreting less sophisticated benchmarks
@@ -515,8 +462,8 @@ robust estimates of the run time of meaningless benchmarks instead.
 
 L<Dumbbench::Instance>,
 L<Dumbbench::Instance::Cmd>,
-L<Dumbbench::Instance::PerlEval>,
-L<Dumbbench::Instance::PerlSub>,
+L<Dumbbench::Instance::PerlEval>, 
+L<Dumbbench::Instance::PerlSub>, 
 L<Dumbbench::Result>
 
 L<Benchmark>
@@ -531,7 +478,7 @@ Steffen Mueller, E<lt>smueller@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2010, 2011, 2012, 2013 by Steffen Mueller
+Copyright (C) 2010 by Steffen Mueller
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.1 or,
